@@ -1,9 +1,13 @@
+import platform
 import warnings
 
+from configobj import ConfigObj, ParseError
 from pgspecial.namedqueries import NamedQueries
+from .config import skip_initial_comment
 
 warnings.filterwarnings("ignore", category=UserWarning, module="psycopg2")
 
+import atexit
 import os
 import re
 import sys
@@ -12,16 +16,19 @@ import logging
 import threading
 import shutil
 import functools
-import humanize
+import pendulum
 import datetime as dt
 import itertools
+import platform
 from time import time, sleep
-from codecs import open
+from typing import Optional
+from urllib.parse import urlparse
 
 keyring = None  # keyring will be loaded later
 
 from cli_helpers.tabular_output import TabularOutputFormatter
 from cli_helpers.tabular_output.preprocessors import align_decimals, format_numbers
+from cli_helpers.utils import strip_ansi
 import click
 
 try:
@@ -58,6 +65,7 @@ from .config import (
     config_location,
     ensure_dir_exists,
     get_config,
+    get_config_filename,
 )
 from .key_bindings import pgcli_bindings
 from .packages.prompt_utils import confirm_destructive_query
@@ -72,14 +80,24 @@ except ImportError:
 
 from getpass import getuser
 from psycopg2 import OperationalError, InterfaceError
+from psycopg2.extensions import make_dsn, parse_dsn
 import psycopg2
 
 from collections import namedtuple
 
 from textwrap import dedent
 
+try:
+    import sshtunnel
+
+    SSH_TUNNEL_SUPPORT = True
+except ImportError:
+    SSH_TUNNEL_SUPPORT = False
+
+
 # Ref: https://stackoverflow.com/questions/30425105/filter-special-chars-such-as-color-codes-from-shell-output
 COLOR_CODE_REGEX = re.compile(r"\x1b(\[.*?[@-~]|\].*?(\x07|\x1b\\))")
+DEFAULT_MAX_FIELD_WIDTH = 500
 
 # Query tuples are used for maintaining history
 MetaQuery = namedtuple(
@@ -100,7 +118,7 @@ MetaQuery.__new__.__defaults__ = ("", False, 0, 0, False, False, False, False)
 
 OutputSettings = namedtuple(
     "OutputSettings",
-    "table_format dcmlfmt floatfmt missingval expanded max_width case_function style_output",
+    "table_format dcmlfmt floatfmt missingval expanded max_width case_function style_output max_field_width",
 )
 OutputSettings.__new__.__defaults__ = (
     None,
@@ -111,6 +129,7 @@ OutputSettings.__new__.__defaults__ = (
     None,
     lambda x: x,
     None,
+    DEFAULT_MAX_FIELD_WIDTH,
 )
 
 
@@ -118,7 +137,7 @@ class PgCliQuitError(Exception):
     pass
 
 
-class PGCli(object):
+class PGCli:
     default_prompt = "\\u@\\h:\\d> "
     max_len_prompt = 30
 
@@ -160,8 +179,8 @@ class PGCli(object):
         prompt_dsn=None,
         auto_vertical_output=False,
         warn=None,
+        ssh_tunnel_url: Optional[str] = None,
     ):
-
         self.force_passwd_prompt = force_passwd_prompt
         self.never_passwd_prompt = never_passwd_prompt
         self.pgexecute = pgexecute
@@ -171,7 +190,11 @@ class PGCli(object):
         # Load config.
         c = self.config = get_config(pgclirc_file)
 
-        NamedQueries.instance = NamedQueries.from_config(self.config)
+        # at this point, config should be written to pgclirc_file if it did not exist. Read it.
+        self.config_writer = load_config(get_config_filename(pgclirc_file))
+
+        # make sure to use self.config_writer, not self.config
+        NamedQueries.instance = NamedQueries.from_config(self.config_writer)
 
         self.logger = logging.getLogger(__name__)
         self.initialize_logging()
@@ -191,14 +214,27 @@ class PGCli(object):
         else:
             self.row_limit = c["main"].as_int("row_limit")
 
+        # if not specified, set to DEFAULT_MAX_FIELD_WIDTH
+        # if specified but empty, set to None to disable truncation
+        # ellipsis will take at least 3 symbols, so this can't be less than 3 if specified and > 0
+        max_field_width = c["main"].get("max_field_width", DEFAULT_MAX_FIELD_WIDTH)
+        if max_field_width and max_field_width.lower() != "none":
+            max_field_width = max(3, abs(int(max_field_width)))
+        else:
+            max_field_width = None
+        self.max_field_width = max_field_width
+
         self.min_num_menu_lines = c["main"].as_int("min_num_menu_lines")
         self.multiline_continuation_char = c["main"]["multiline_continuation_char"]
         self.table_format = c["main"]["table_format"]
         self.syntax_style = c["main"]["syntax_style"]
         self.cli_style = c["colors"]
         self.wider_completion_menu = c["main"].as_bool("wider_completion_menu")
-        c_dest_warning = c["main"].as_bool("destructive_warning")
-        self.destructive_warning = c_dest_warning if warn is None else warn
+        self.destructive_warning = warn or c["main"]["destructive_warning"]
+        # also handle boolean format of destructive warning
+        self.destructive_warning = {"true": "all", "false": "off"}.get(
+            self.destructive_warning.lower(), self.destructive_warning
+        )
         self.less_chatty = bool(less_chatty) or c["main"].as_bool("less_chatty")
         self.null_string = c["main"].get("null_string", "<null>")
         self.prompt_format = (
@@ -211,6 +247,7 @@ class PGCli(object):
         self.decimal_format = c["data_formats"]["decimal"]
         self.float_format = c["data_formats"]["float"]
         self.initialize_keyring()
+        self.show_bottom_toolbar = c["main"].as_bool("show_bottom_toolbar")
 
         self.pgspecial.pset_pager(
             self.config["main"].as_bool("enable_pager") and "on" or "off"
@@ -249,6 +286,10 @@ class PGCli(object):
 
         self.prompt_app = None
 
+        self.ssh_tunnel_config = c.get("ssh tunnels")
+        self.ssh_tunnel_url = ssh_tunnel_url
+        self.ssh_tunnel = None
+
     def quit(self):
         raise PgCliQuitError
 
@@ -320,11 +361,11 @@ class PGCli(object):
             if pattern not in TabularOutputFormatter().supported_formats:
                 raise ValueError()
             self.table_format = pattern
-            yield (None, None, None, "Changed table format to {}".format(pattern))
+            yield (None, None, None, f"Changed table format to {pattern}")
         except ValueError:
-            msg = "Table format {} not recognized. Allowed formats:".format(pattern)
+            msg = f"Table format {pattern} not recognized. Allowed formats:"
             for table_type in TabularOutputFormatter().supported_formats:
-                msg += "\n\t{}".format(table_type)
+                msg += f"\n\t{table_type}"
             msg += "\nCurrently set to: %s" % self.table_format
             yield (None, None, None, msg)
 
@@ -381,10 +422,13 @@ class PGCli(object):
         try:
             with open(os.path.expanduser(pattern), encoding="utf-8") as f:
                 query = f.read()
-        except IOError as e:
+        except OSError as e:
             return [(None, None, None, str(e), "", False, True)]
 
-        if self.destructive_warning and confirm_destructive_query(query) is False:
+        if (
+            self.destructive_warning != "off"
+            and confirm_destructive_query(query, self.destructive_warning) is False
+        ):
             message = "Wise choice. Command execution stopped."
             return [(None, None, None, message)]
 
@@ -402,7 +446,7 @@ class PGCli(object):
         if not os.path.isfile(filename):
             try:
                 open(filename, "w").close()
-            except IOError as e:
+            except OSError as e:
                 self.output_file = None
                 message = str(e) + "\nFile output disabled"
                 return [(None, None, None, message, "", False, True)]
@@ -470,6 +514,21 @@ class PGCli(object):
     def connect_dsn(self, dsn, **kwargs):
         self.connect(dsn=dsn, **kwargs)
 
+    def connect_service(self, service, user):
+        service_config, file = parse_service_info(service)
+        if service_config is None:
+            click.secho(
+                f"service '{service}' was not found in {file}", err=True, fg="red"
+            )
+            exit(1)
+        self.connect(
+            database=service_config.get("dbname"),
+            host=service_config.get("host"),
+            user=user or service_config.get("user"),
+            port=service_config.get("port"),
+            passwd=service_config.get("password"),
+        )
+
     def connect_uri(self, uri):
         kwargs = psycopg2.extensions.parse_dsn(uri)
         remap = {"dbname": "database", "password": "passwd"}
@@ -495,7 +554,7 @@ class PGCli(object):
             passwd = os.environ.get("PGPASSWORD", "")
 
         # Find password from store
-        key = "%s@%s" % (user, host)
+        key = f"{user}@{host}"
         keyring_error_message = dedent(
             """\
             {}
@@ -505,6 +564,17 @@ class PGCli(object):
             - uninstall keyring: pip uninstall keyring
             - disable keyring in our configuration: add keyring = False to [main]"""
         )
+
+        # Prompt for a password immediately if requested via the -W flag. This
+        # avoids wasting time trying to connect to the database and catching a
+        # no-password exception.
+        # If we successfully parsed a password from a URI, there's no need to
+        # prompt for it, even with the -W flag
+        if self.force_passwd_prompt and not passwd:
+            passwd = click.prompt(
+                "Password for %s" % user, hide_input=True, show_default=False, type=str
+            )
+
         if not passwd and keyring:
 
             try:
@@ -518,16 +588,6 @@ class PGCli(object):
                     fg="red",
                 )
 
-        # Prompt for a password immediately if requested via the -W flag. This
-        # avoids wasting time trying to connect to the database and catching a
-        # no-password exception.
-        # If we successfully parsed a password from a URI, there's no need to
-        # prompt for it, even with the -W flag
-        if self.force_passwd_prompt and not passwd:
-            passwd = click.prompt(
-                "Password for %s" % user, hide_input=True, show_default=False, type=str
-            )
-
         def should_ask_for_password(exc):
             # Prompt for a password after 1st attempt to connect
             # fails. Don't prompt if the -w flag is supplied
@@ -540,6 +600,56 @@ class PGCli(object):
                 return True
             return False
 
+        if dsn:
+            parsed_dsn = parse_dsn(dsn)
+            if "host" in parsed_dsn:
+                host = parsed_dsn["host"]
+            if "port" in parsed_dsn:
+                port = parsed_dsn["port"]
+
+        if self.ssh_tunnel_config and not self.ssh_tunnel_url:
+            for db_host_regex, tunnel_url in self.ssh_tunnel_config.items():
+                if re.search(db_host_regex, host):
+                    self.ssh_tunnel_url = tunnel_url
+                    break
+
+        if self.ssh_tunnel_url:
+            # We add the protocol as urlparse doesn't find it by itself
+            if "://" not in self.ssh_tunnel_url:
+                self.ssh_tunnel_url = f"ssh://{self.ssh_tunnel_url}"
+
+            tunnel_info = urlparse(self.ssh_tunnel_url)
+            params = {
+                "local_bind_address": ("127.0.0.1",),
+                "remote_bind_address": (host, int(port or 5432)),
+                "ssh_address_or_host": (tunnel_info.hostname, tunnel_info.port or 22),
+                "logger": self.logger,
+            }
+            if tunnel_info.username:
+                params["ssh_username"] = tunnel_info.username
+            if tunnel_info.password:
+                params["ssh_password"] = tunnel_info.password
+
+            # Hack: sshtunnel adds a console handler to the logger, so we revert handlers.
+            # We can remove this when https://github.com/pahaz/sshtunnel/pull/250 is merged.
+            logger_handlers = self.logger.handlers.copy()
+            try:
+                self.ssh_tunnel = sshtunnel.SSHTunnelForwarder(**params)
+                self.ssh_tunnel.start()
+            except Exception as e:
+                self.logger.handlers = logger_handlers
+                self.logger.error("traceback: %r", traceback.format_exc())
+                click.secho(str(e), err=True, fg="red")
+                exit(1)
+            self.logger.handlers = logger_handlers
+
+            atexit.register(self.ssh_tunnel.stop)
+            host = "127.0.0.1"
+            port = self.ssh_tunnel.local_bind_ports[0]
+
+            if dsn:
+                dsn = make_dsn(dsn, host=host, port=port)
+
         # Attempt to connect to the database.
         # Note that passwd may be empty on the first attempt. If connection
         # fails because of a missing or incorrect password, but we're allowed to
@@ -624,8 +734,10 @@ class PGCli(object):
         query = MetaQuery(query=text, successful=False)
 
         try:
-            if self.destructive_warning:
-                destroy = confirm = confirm_destructive_query(text)
+            if self.destructive_warning != "off":
+                destroy = confirm = confirm_destructive_query(
+                    text, self.destructive_warning
+                )
                 if destroy is False:
                     click.secho("Wise choice!")
                     raise KeyboardInterrupt
@@ -657,10 +769,11 @@ class PGCli(object):
                             click.echo(text, file=f)
                             click.echo("\n".join(output), file=f)
                             click.echo("", file=f)  # extra newline
-                    except IOError as e:
+                    except OSError as e:
                         click.secho(str(e), err=True, fg="red")
                 else:
-                    self.echo_via_pager("\n".join(output))
+                    if output:
+                        self.echo_via_pager("\n".join(output))
             except KeyboardInterrupt:
                 pass
 
@@ -671,9 +784,9 @@ class PGCli(object):
                         "Time: %0.03fs (%s), executed in: %0.03fs (%s)"
                         % (
                             query.total_time,
-                            humanize.time.naturaldelta(query.total_time),
+                            pendulum.Duration(seconds=query.total_time).in_words(),
                             query.execution_time,
-                            humanize.time.naturaldelta(query.execution_time),
+                            pendulum.Duration(seconds=query.execution_time).in_words(),
                         )
                     )
                 else:
@@ -708,7 +821,6 @@ class PGCli(object):
         if not self.less_chatty:
             print("Server: PostgreSQL", self.pgexecute.server_version)
             print("Version:", __version__)
-            print("Chat: https://gitter.im/dbcli/pgcli")
             print("Home: http://pgcli.com")
 
         try:
@@ -726,22 +838,7 @@ class PGCli(object):
                     click.secho(str(e), err=True, fg="red")
                     continue
 
-                # Initialize default metaquery in case execution fails
-                self.watch_command, timing = special.get_watch_command(text)
-                if self.watch_command:
-                    while self.watch_command:
-                        try:
-                            query = self.execute_command(self.watch_command)
-                            click.echo(
-                                "Waiting for {0} seconds before repeating".format(
-                                    timing
-                                )
-                            )
-                            sleep(timing)
-                        except KeyboardInterrupt:
-                            self.watch_command = None
-                else:
-                    query = self.execute_command(text)
+                self.handle_watch_command(text)
 
                 self.now = dt.datetime.today()
 
@@ -749,12 +846,40 @@ class PGCli(object):
                 with self._completer_lock:
                     self.completer.extend_query_history(text)
 
-                self.query_history.append(query)
-
         except (PgCliQuitError, EOFError):
             if not self.less_chatty:
                 print("Goodbye!")
 
+    def handle_watch_command(self, text):
+        # Initialize default metaquery in case execution fails
+        self.watch_command, timing = special.get_watch_command(text)
+
+        # If we run \watch without a command, apply it to the last query run.
+        if self.watch_command is not None and not self.watch_command.strip():
+            try:
+                self.watch_command = self.query_history[-1].query
+            except IndexError:
+                click.secho(
+                    "\\watch cannot be used with an empty query", err=True, fg="red"
+                )
+                self.watch_command = None
+
+        # If there's a command to \watch, run it in a loop.
+        if self.watch_command:
+            while self.watch_command:
+                try:
+                    query = self.execute_command(self.watch_command)
+                    click.echo(f"Waiting for {timing} seconds before repeating")
+                    sleep(timing)
+                except KeyboardInterrupt:
+                    self.watch_command = None
+
+        # Otherwise, execute it as a regular command.
+        else:
+            query = self.execute_command(text)
+
+        self.query_history.append(query)
+
     def _build_cli(self, history):
         key_bindings = pgcli_bindings(self)
 
@@ -792,7 +917,7 @@ class PGCli(object):
                 reserve_space_for_menu=self.min_num_menu_lines,
                 message=get_message,
                 prompt_continuation=get_continuation,
-                bottom_toolbar=get_toolbar_tokens,
+                bottom_toolbar=get_toolbar_tokens if self.show_bottom_toolbar else None,
                 complete_style=complete_style,
                 input_processors=[
                     # Highlight matching brackets while editing.
@@ -904,6 +1029,7 @@ class PGCli(object):
                     else lambda x: x
                 ),
                 style_output=self.style_output,
+                max_field_width=self.max_field_width,
             )
             execution = time() - start
             formatted = format_output(title, cur, headers, status, settings)
@@ -947,7 +1073,7 @@ class PGCli(object):
             click.secho(str(e), err=True, fg="red")
 
     def refresh_completions(self, history=None, persist_priorities="all"):
-        """ Refresh outdated completions
+        """Refresh outdated completions
 
         :param history: A prompt_toolkit.history.FileHistory object. Used to
                         load keyword and identifier preferences
@@ -958,16 +1084,13 @@ class PGCli(object):
         callback = functools.partial(
             self._on_completions_refreshed, persist_priorities=persist_priorities
         )
-        self.completion_refresher.refresh(
+        return self.completion_refresher.refresh(
             self.pgexecute,
             self.pgspecial,
             callback,
             history=history,
             settings=self.settings,
         )
-        return [
-            (None, None, None, "Auto-completion refresh started in the background.")
-        ]
 
     def _on_completions_refreshed(self, new_completer, persist_priorities):
         self._swap_completer_objects(new_completer, persist_priorities)
@@ -1028,7 +1151,7 @@ class PGCli(object):
             str(self.pgexecute.port) if self.pgexecute.port is not None else "5432",
         )
         string = string.replace("\\i", str(self.pgexecute.pid) or "(none)")
-        string = string.replace("\\#", "#" if (self.pgexecute.superuser) else ">")
+        string = string.replace("\\#", "#" if self.pgexecute.superuser else ">")
         string = string.replace("\\n", "\n")
         return string
 
@@ -1054,7 +1177,10 @@ class PGCli(object):
     def echo_via_pager(self, text, color=None):
         if self.pgspecial.pager_config == PAGER_OFF or self.watch_command:
             click.echo(text, color=color)
-        elif self.pgspecial.pager_config == PAGER_LONG_OUTPUT:
+        elif (
+            self.pgspecial.pager_config == PAGER_LONG_OUTPUT
+            and self.table_format != "csv"
+        ):
             lines = text.split("\n")
 
             # The last 4 lines are reserved for the pgcli menu and padding
@@ -1161,7 +1287,7 @@ class PGCli(object):
     "--list",
     "list_databases",
     is_flag=True,
-    help="list " "available databases, then exit.",
+    help="list available databases, then exit.",
 )
 @click.option(
     "--auto-vertical-output",
@@ -1169,7 +1295,15 @@ class PGCli(object):
     help="Automatically switch to vertical output mode if the result is wider than the terminal width.",
 )
 @click.option(
-    "--warn/--no-warn", default=None, help="Warn before running a destructive query."
+    "--warn",
+    default=None,
+    type=click.Choice(["all", "moderate", "off"]),
+    help="Warn before running a destructive query.",
+)
+@click.option(
+    "--ssh-tunnel",
+    default=None,
+    help="Open an SSH tunnel to the given address and connect to the database from it.",
 )
 @click.argument("dbname", default=lambda: None, envvar="PGDATABASE", nargs=1)
 @click.argument("username", default=lambda: None, envvar="PGUSER", nargs=1)
@@ -1194,6 +1328,7 @@ def cli(
     auto_vertical_output,
     list_dsn,
     warn,
+    ssh_tunnel: str,
 ):
     if version:
         print("Version:", __version__)
@@ -1212,7 +1347,8 @@ def cli(
         else:
             print("Config file is now located at", config_full_path)
             print(
-                "Please move the existing config file ~/.pgclirc to", config_full_path,
+                "Please move the existing config file ~/.pgclirc to",
+                config_full_path,
             )
     if list_dsn:
         try:
@@ -1229,6 +1365,15 @@ def cli(
             )
             exit(1)
 
+    if ssh_tunnel and not SSH_TUNNEL_SUPPORT:
+        click.secho(
+            'Cannot open SSH tunnel, "sshtunnel" package was not found. '
+            "Please install pgcli with `pip install pgcli[sshtunnel]` if you want SSH tunnel support.",
+            err=True,
+            fg="red",
+        )
+        exit(1)
+
     pgcli = PGCli(
         prompt_passwd,
         never_prompt,
@@ -1240,6 +1385,7 @@ def cli(
         prompt_dsn=prompt_dsn,
         auto_vertical_output=auto_vertical_output,
         warn=warn,
+        ssh_tunnel_url=ssh_tunnel,
     )
 
     # Choose which ever one has a valid value.
@@ -1248,7 +1394,11 @@ def cli(
         username = dbname
     database = dbname_opt or dbname or ""
     user = username_opt or username
-
+    service = None
+    if database.startswith("service="):
+        service = database[8:]
+    elif os.getenv("PGSERVICE") is not None:
+        service = os.getenv("PGSERVICE")
     # because option --list or -l are not supposed to have a db name
     if list_databases:
         database = "postgres"
@@ -1257,7 +1407,15 @@ def cli(
         try:
             cfg = load_config(pgclirc, config_full_path)
             dsn_config = cfg["alias_dsn"][dsn]
-        except:
+        except KeyError:
+            click.secho(
+                f"Could not find a DSN with alias {dsn}. "
+                'Please check the "[alias_dsn]" section in pgclirc.',
+                err=True,
+                fg="red",
+            )
+            exit(1)
+        except Exception:
             click.secho(
                 "Invalid DSNs found in the config file. "
                 'Please check the "[alias_dsn]" section in pgclirc.',
@@ -1269,10 +1427,10 @@ def cli(
         pgcli.dsn_alias = dsn
     elif "://" in database:
         pgcli.connect_uri(database)
-    elif "=" in database:
+    elif "=" in database and service is None:
         pgcli.connect_dsn(database, user=user)
-    elif os.environ.get("PGSERVICE", None):
-        pgcli.connect_dsn("service={0}".format(os.environ["PGSERVICE"]))
+    elif service is not None:
+        pgcli.connect_service(service, user)
     else:
         pgcli.connect(database, host, user, port)
 
@@ -1348,7 +1506,7 @@ def is_mutating(status):
     if not status:
         return False
 
-    mutating = set(["insert", "update", "delete"])
+    mutating = {"insert", "update", "delete"}
     return status.split(None, 1)[0].lower() in mutating
 
 
@@ -1398,10 +1556,19 @@ def format_output(title, cur, headers, status, settings):
         "disable_numparse": True,
         "preserve_whitespace": True,
         "style": settings.style_output,
+        "max_field_width": settings.max_field_width,
     }
     if not settings.floatfmt:
         output_kwargs["preprocessors"] = (align_decimals,)
 
+    if table_format == "csv":
+        # The default CSV dialect is "excel" which is not handling newline values correctly
+        # Nevertheless, we want to keep on using "excel" on Windows since it uses '\r\n'
+        # as the line terminator
+        # https://github.com/dbcli/pgcli/issues/1102
+        dialect = "excel" if platform.system() == "Windows" else "unix"
+        output_kwargs["dialect"] = dialect
+
     if title:  # Only print the title if it's not None.
         output.append(title)
 
@@ -1431,7 +1598,12 @@ def format_output(title, cur, headers, status, settings):
             formatted = iter(formatted.splitlines())
         first_line = next(formatted)
         formatted = itertools.chain([first_line], formatted)
-        if not expanded and max_width and len(first_line) > max_width and headers:
+        if (
+            not expanded
+            and max_width
+            and len(strip_ansi(first_line)) > max_width
+            and headers
+        ):
             formatted = formatter.format_output(
                 cur, headers, format_name="vertical", column_types=None, **output_kwargs
             )
@@ -1440,11 +1612,39 @@ def format_output(title, cur, headers, status, settings):
 
         output = itertools.chain(output, formatted)
 
-    if status:  # Only print the status if it's not None.
+    # Only print the status if it's not None and we are not producing CSV
+    if status and table_format != "csv":
         output = itertools.chain(output, [status])
 
     return output
 
 
+def parse_service_info(service):
+    service = service or os.getenv("PGSERVICE")
+    service_file = os.getenv("PGSERVICEFILE")
+    if not service_file:
+        # try ~/.pg_service.conf (if that exists)
+        if platform.system() == "Windows":
+            service_file = os.getenv("PGSYSCONFDIR") + "\\pg_service.conf"
+        elif os.getenv("PGSYSCONFDIR"):
+            service_file = os.path.join(os.getenv("PGSYSCONFDIR"), ".pg_service.conf")
+        else:
+            service_file = os.path.expanduser("~/.pg_service.conf")
+    if not service or not os.path.exists(service_file):
+        # nothing to do
+        return None, service_file
+    with open(service_file, newline="") as f:
+        skipped_lines = skip_initial_comment(f)
+        try:
+            service_file_config = ConfigObj(f)
+        except ParseError as err:
+            err.line_number += skipped_lines
+            raise err
+    if service not in service_file_config:
+        return None, service_file
+    service_conf = service_file_config.get(service)
+    return service_conf, service_file
+
+
 if __name__ == "__main__":
     cli()
